﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HtmlAgilityPack;
using Quartz;
using SCC.Common;
using SCC.Crawler.Tools;
using SCC.Interface;
using SCC.Models;

namespace SCC.Crawler.DT
{
    /// <summary>
    ///     黑龙江P62
    /// </summary>
    [DisallowConcurrentExecution]
    [PersistJobDataAfterExecution]
    public class HeiLongJiangP62Job : IJob
    {
        public HeiLongJiangP62Job()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
            email = IOC.Resolve<IEmail>();
        }

        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);


            var UrlList = GetKaiJiangUrl(Config.Number);
            foreach (var list in UrlList)
            {
                var OpenList = GetOpenListFromMainUrl(list.Url);
                foreach (var openList in OpenList)
                {
                    LatestItem = openList;
                    if (LatestItem != null && services.AddDTOpen7Code(currentLottery, LatestItem))
                    {
                        log.Info(GetType(), CommonHelper.GetJobMainLogInfo(Config, openList.Term.ToString()));
                        isGetData = true;
                    }
                }
                context.JobDetail.JobDataMap["LatestItem"] = LatestItem;
            }

        }
        private List<Periods> GetKaiJiangUrl(string Number)
        {
            string MianUrl = Config.MainUrl;
            List<Periods> list = new List<Periods>();
            for (int i = 1; i <= int.Parse(Number); i++)
            {
                Periods ssq = new Periods();
                if (i == 1)
                    ssq.Url = MianUrl + "/heilongjiang/static/html/p62/list/list_p62_1.html";
                else
                    ssq.Url = MianUrl + "/heilongjiang/static/html/p62/list/list_p62_" + i+ ".html";
                list.Add(ssq);
            }
            return list;
        }
        private List<OpenCode7DTModel> GetOpenListFromMainUrl(string  MainUrl)
        {
            var result = new List<OpenCode7DTModel>();
            try
            {
                var htmlResource = NetHelper.GetUrlResponse(MainUrl, Encoding.GetEncoding("utf-8"));
                if (htmlResource == null) return result;

                var doc = new HtmlDocument();
                doc.LoadHtml(htmlResource);
                var table = doc.DocumentNode.SelectSingleNode("//table");
                if (table == null) return result;
                var trs = table.ChildNodes.Where(node => node.Name == "tr").ToList();
                List<HtmlNode> tds = null;
                string term = string.Empty, openCodeString = string.Empty, optimizeUrl = string.Empty;
                OpenCode7DTModel model = null;
                for (var i = 0; i < trs.Count; i++)
                {
                    tds = trs[i].ChildNodes.Where(S => S.Name.ToLower() == "td").ToList();
                    if (tds.Count < 4) continue;

                    model = new OpenCode7DTModel();
                    term = tds[0].InnerText.Trim();
                    model.OpenTime = Convert.ToDateTime(tds[1].InnerText.Trim());
                    model.Term = Convert.ToInt64(term);
                    openCodeString = tds[2].InnerText.Replace("&nbsp;", "").Replace("\t\t    ", "").Trim();
                    model.OpenCode1 = Convert.ToInt32(openCodeString.Substring(0, 1));
                    model.OpenCode2 = Convert.ToInt32(openCodeString.Substring(1, 1));
                    model.OpenCode3 = Convert.ToInt32(openCodeString.Substring(2, 1));
                    model.OpenCode4 = Convert.ToInt32(openCodeString.Substring(3, 1));
                    model.OpenCode5 = Convert.ToInt32(openCodeString.Substring(4, 1));
                    model.OpenCode6 = Convert.ToInt32(openCodeString.Substring(5, 1));
                    model.OpenCode7 = Convert.ToInt32(openCodeString.Substring(6, 1));

                    //组装开奖详情
                    result.Add(model);
                }

                var checkDataHelper = new CheckDataHelper();
                var dbdata = services.GetListS<OpenCode7DTModel>(currentLottery)
                    .ToDictionary(w => w.Term.ToString(), w => w.GetCodeStr());
                checkDataHelper.CheckData(dbdata, result.ToDictionary(w => w.Term.ToString(), w => w.GetCodeStr()),
                    Config.Area, currentLottery);
            }
            catch (Exception ex)
            {
                log.Error(GetType(),
                    string.Format("【{0}】通过备用站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + currentLottery, ex.Message));
            }

            return result;
        }



        /// <summary>
        ///     配置信息
        /// </summary>
        private SCCConfig Config;

        /// <summary>
        ///     当天抓取的最新一期开奖记录
        /// </summary>
        private OpenCode7DTModel LatestItem;

        /// <summary>
        ///     当天抓取失败列表
        /// </summary>
        private List<string> FailedQiHaoList = null;

        /// <summary>
        ///     日志对象
        /// </summary>
        private readonly LogHelper log;

        /// <summary>
        ///     数据服务
        /// </summary>
        private readonly IDTOpenCode services;

        /// <summary>
        ///     当前彩种
        /// </summary>
        private SCCLottery currentLottery => SCCLottery.HeiLongJiangP62;

        /// <summary>
        ///     邮件接口
        /// </summary>
        private IEmail email;

        /// <summary>
        ///     是否本次运行抓取到开奖数据
        /// </summary>
        private bool isGetData;

    
    }
}