﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HtmlAgilityPack;
using Quartz;
using SCC.Common;
using SCC.Crawler.Tools;
using SCC.Interface;
using SCC.Models;

namespace SCC.Crawler.DT
{
    /// <summary>
    ///     黑龙江体彩6+1
    /// </summary>
    [DisallowConcurrentExecution]
    [PersistJobDataAfterExecution]
    public class HeiLongJiangTC6J1Job : IJob
    {
        public HeiLongJiangTC6J1Job()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
            email = IOC.Resolve<IEmail>();
        }

        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);


            var UrlList = GetKaiJiangUrl(Config.Number);
            foreach (var list in UrlList)
            {
                var OpenList = GetOpenListFromMainUrl(list.Url);
                foreach (var openList in OpenList)
                {
                    LatestItem = openList;
                    if (LatestItem != null && services.AddDTOpen7Code(currentLottery, LatestItem))
                    {
                        log.Info(GetType(), CommonHelper.GetJobMainLogInfo(Config, openList.Term.ToString()));
                        isGetData = true;
                    }
                }
                context.JobDetail.JobDataMap["LatestItem"] = LatestItem;
            }

        }
        private List<Periods> GetKaiJiangUrl(string Number)
        {
            string MianUrl = Config.MainUrl;
            List<Periods> list = new List<Periods>();
            for (int i = 1; i <= int.Parse(Number); i++)
            {
                Periods ssq = new Periods();
                if (i == 1)
                    ssq.Url = MianUrl + "historykj/history.jspx?_ltype=hlj61";
                else
                    ssq.Url = MianUrl + "historykj/history_" + i + ".jspx?_ltype=hlj61";
                list.Add(ssq);
            }
            return list;
        }
        /// <summary>
        ///     爬取主网站信息
        /// </summary>
        /// <returns></returns>
        private List<OpenCode7DTModel> GetOpenListFromMainUrl(string mainUrl)
        {
            var result = new List<OpenCode7DTModel>();
            try
            {
                var htmlResource = NetHelper.GetUrlResponse(mainUrl, Encoding.GetEncoding("utf-8"));
                if (htmlResource == null) return result;

                var doc = new HtmlDocument();
                doc.LoadHtml(htmlResource);
                var table = doc.DocumentNode.SelectNodes("//tbody");
                if (table == null) return result;
                var trs = table[0].ChildNodes.Where(S => S.Name.ToLower() == "tr").ToList();
                List<HtmlNode> tds = null;
                string term = string.Empty, openCodeString = string.Empty, optimizeUrl = string.Empty;
                OpenCode7DTModel model = null;
                for (var i = 0; i < trs.Count; i++)
                {
                    tds = trs[i].ChildNodes.Where(S => S.Name.ToLower() == "td").ToList();
                    if (tds.Count < 10) continue;

                    model = new OpenCode7DTModel();
                    term = tds[0].InnerText.Trim();
            
                    model.Term = Convert.ToInt64(term);
                    openCodeString = tds[1].InnerText.Trim();
                    model.OpenCode1 = Convert.ToInt32(openCodeString.Substring(0, 2));
                    model.OpenCode2 = Convert.ToInt32(openCodeString.Substring(2, 2));
                    model.OpenCode3 = Convert.ToInt32(openCodeString.Substring(4, 2));
                    model.OpenCode4 = Convert.ToInt32(openCodeString.Substring(6, 2));
                    model.OpenCode5 = Convert.ToInt32(openCodeString.Substring(8, 2));
                    model.OpenCode6 = Convert.ToInt32(openCodeString.Substring(10, 2));
                    model.OpenCode7 = Convert.ToInt32(openCodeString.Substring(13, 2));
                    model.OpenTime = Convert.ToDateTime(tds[10].InnerText.Trim());
                    //组装开奖详情
                    var details = GetKaijiangDetails(tds);
                    model.Spare = details;
                    model.DetailUrl = mainUrl;

                    result.Add(model);
                }

                var checkDataHelper = new CheckDataHelper();
                var dbdata = services.GetListS<OpenCode7DTModel>(currentLottery)
                    .ToDictionary(w => w.Term.ToString(), w => w.GetCodeStr());
                checkDataHelper.CheckData(dbdata, result.ToDictionary(w => w.Term.ToString(), w => w.GetCodeStr()),
                    Config.Area, currentLottery);
            }
            catch (Exception ex)
            {
                log.Error(GetType(),
                    string.Format("【{0}】通过主站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + currentLottery, ex.Message));
            }

            return result;
        }

    
        private string GetKaijiangDetails(List<HtmlNode> nodes)
        {
            var entity = new KaijiangDetailsEntity
            {
                Gdje = nodes[9].InnerText.Replace(",", ""),
                Trje = nodes[8].InnerText.Replace(",", "")
            };
            //TODO 
            var list1 = new List<Kaijiangitem>();
            //组装详情
            var list = new List<Kaijiangitem>
            {
                new Kaijiangitem
                {
                    Name = "特等奖",
                    /// "0注<br>\t\t\t\t\t\t\t0.00元"
                    Total = nodes[2].InnerText.Replace("<br>", "").Replace("\t", "").Replace("元", "")
                        .Split("注".ToCharArray())[0],
                    TotalMoney =
                        nodes[2].InnerText.Replace("<br>", "").Replace("\t", "").Replace("元", "")
                            .Split("注".ToCharArray())[1].Replace(",", "")
                },
                new Kaijiangitem
                {
                    Name = "一等奖",
                    /// "0注<br>\t\t\t\t\t\t\t0.00元"
                    Total = nodes[3].InnerText.Replace("<br>", "").Replace("\t", "").Replace("元", "")
                        .Split("注".ToCharArray())[0],
                    TotalMoney =
                        nodes[3].InnerText.Replace("<br>", "").Replace("\t", "").Replace("元", "")
                            .Split("注".ToCharArray())[1].Replace(",", "")
                },
                new Kaijiangitem
                {
                    Name = "二等奖",
                    /// "0注<br>\t\t\t\t\t\t\t0.00元"
                    Total = nodes[4].InnerText.Replace("<br>", "").Replace("\t", "").Replace("元", "")
                        .Split("注".ToCharArray())[0],
                    TotalMoney =
                        nodes[4].InnerText.Replace("<br>", "").Replace("\t", "").Replace("元", "")
                            .Split("注".ToCharArray())[1].Replace(",", "")
                },
                new Kaijiangitem
                {
                    Name = "三等奖",
                    /// "0注<br>\t\t\t\t\t\t\t0.00元"
                    Total = nodes[4].InnerText.Replace("<br>", "").Replace("\t", "").Replace("元", "")
                        .Split("注".ToCharArray())[0],
                    TotalMoney =
                        nodes[4].InnerText.Replace("<br>", "").Replace("\t", "").Replace("元", "")
                            .Split("注".ToCharArray())[1]
                },
                new Kaijiangitem
                {
                    Name = "四等奖",
                    /// "0注<br>\t\t\t\t\t\t\t0.00元"
                    Total = nodes[5].InnerText.Replace("<br>", "").Replace("\t", "").Replace("元", "")
                        .Split("注".ToCharArray())[0],
                    TotalMoney =
                        nodes[5].InnerText.Replace("<br>", "").Replace("\t", "").Replace("元", "")
                            .Split("注".ToCharArray())[1]
                },
                new Kaijiangitem
                {
                    Name = "五等奖",
                    /// "0注<br>\t\t\t\t\t\t\t0.00元"
                    Total = nodes[2].InnerText.Replace("<br>", "").Replace("\t", "").Replace("元", "")
                        .Split("注".ToCharArray())[0],
                    TotalMoney =
                        nodes[2].InnerText.Replace("<br>", "").Replace("\t", "").Replace("元", "")
                            .Split("注".ToCharArray())[1]
                }
            };
            entity.KaiJiangItems = list;

            return entity.TryToJson();
        }

        #region Attribute

        /// <summary>
        ///     配置信息
        /// </summary>
        private SCCConfig Config;

        /// <summary>
        ///     当天抓取的最新一期开奖记录
        /// </summary>
        private OpenCode7DTModel LatestItem;

        /// <summary>
        ///     当天抓取失败列表
        /// </summary>
        private List<string> FailedQiHaoList = null;

        /// <summary>
        ///     日志对象
        /// </summary>
        private readonly LogHelper log;

        /// <summary>
        ///     数据服务
        /// </summary>
        private readonly IDTOpenCode services;

        /// <summary>
        ///     当前彩种
        /// </summary>
        private SCCLottery currentLottery => SCCLottery.HeiLongJiangTC6J1;

        /// <summary>
        ///     邮件接口
        /// </summary>
        private IEmail email;

        /// <summary>
        ///     是否本次运行抓取到开奖数据
        /// </summary>
        private bool isGetData;

        #endregion
    }
}