﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HtmlAgilityPack;
using Quartz;
using SCC.Common;
using SCC.Crawler.Tools;
using SCC.Interface;
using SCC.Models;

namespace SCC.Crawler.DT
{
    /// <summary>
    ///     香港六合彩
    /// </summary>
    [DisallowConcurrentExecution]
    [PersistJobDataAfterExecution]
    public class HongKongSMHLHCJob : IJob
    {
        public HongKongSMHLHCJob()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
            email = IOC.Resolve<IEmail>();
        }


        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);


            var UrlList = GetNumberList();
            foreach (var list in UrlList)
            {
                var OpenList = GetOpenListFromMainUrl(list.Year);
                foreach (var openList in OpenList)
                {
                    LatestItem = openList;
                    if (LatestItem != null && services.AddDTOpen7Code(currentLottery, LatestItem))
                    {
                        log.Info(GetType(), CommonHelper.GetJobMainLogInfo(Config, openList.Term.ToString()));
                        isGetData = true;
                    }

                }
                ////保存最新期号
                context.JobDetail.JobDataMap["LatestItem"] = LatestItem;
            }
        }


        public List<Periods> GetNumberList()
        {
            List<Periods> list = new List<Periods>();
            int StartYear = 2010;
            int AtLastYear = 2018;
            for (int i = StartYear; i <= AtLastYear; i++)
            {
                Periods ssq = new Periods();
                ssq.Year = i;
                list.Add(ssq);
            }
            return list;
        }
        private List<OpenCode7DTModel> GetOpenListFromMainUrl(int Year)
        {
            var result = new List<OpenCode7DTModel>();
            try
            {
                var URL = "https://www.06266.com/06266/history/" + Year + ".html";
      
                var htmlResource = NetHelper.GetUrlResponse(URL, Encoding.GetEncoding("UTF-8"));
                if (htmlResource == null) return result;

                var doc = new HtmlDocument();
                doc.LoadHtml(htmlResource);
                var div = doc.DocumentNode.SelectSingleNode("//div[@class='bd']");
                if (div == null) return result;

                var table = div.ChildNodes.Where(node => node.Name == "table").ToList();
                if (table == null) return result;
                var trs = table[0].ChildNodes.Where(node => node.Name == "tr").ToList();
                OpenCode7DTModel model = null;
                var optimizeUrl = string.Empty;
                for (var i = 1; i < trs.Count; i++) //第一二行为表头
                {
                    var tds = trs[i].ChildNodes.Where(node => node.Name == "td").ToList();
                    if (tds.Count < 23) continue;
                    model = new OpenCode7DTModel();
                    var QiHao = tds[0].InnerText.Substring(0, 4) + tds[1].InnerText.Trim().Substring(0, 3);
                    model.Term = Convert.ToInt64(QiHao);

                    model.OpenTime = Convert.ToDateTime(tds[0].InnerText);
                    var openCodeString = tds[2].InnerText.Trim();

                    model.OpenCode1 = Convert.ToInt32(openCodeString.Substring(0, 2));
                    model.OpenCode2 = Convert.ToInt32(openCodeString.Substring(3, 2));
                    model.OpenCode3 = Convert.ToInt32(openCodeString.Substring(6, 2));
                    model.OpenCode4 = Convert.ToInt32(openCodeString.Substring(9, 2));
                    model.OpenCode5 = Convert.ToInt32(openCodeString.Substring(12, 2));
                    model.OpenCode6 = Convert.ToInt32(openCodeString.Substring(15, 2));
                    model.OpenCode7 = Convert.ToInt32(tds[3].InnerText.Trim().Substring(0, 2));
                    model.Spare = string.Empty;
                    if (result.SingleOrDefault(w => w.Term == model.Term) == null) result.Add(model);
                }
                
                result = result.OrderByDescending(S => S.Term).ToList();
            }
            catch (Exception ex)
            {
                log.Error(GetType(),
                    string.Format("【{0}】通过主站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + currentLottery, ex.Message));
            }

            return result;
        }


        /// <param name="model"></param>
        /// <summary>
        ///     配置信息
        /// </summary>
        private SCCConfig Config;

        /// <summary>
        ///     当天抓取的最新一期开奖记录
        /// </summary>
        private OpenCode7DTModel LatestItem;

        /// <summary>
        ///     当天抓取失败列表
        /// </summary>
        private List<string> FailedQiHaoList = null;

        /// <summary>
        ///     日志对象
        /// </summary>
        private readonly LogHelper log;

        /// <summary>
        ///     数据服务
        /// </summary>
        private readonly IDTOpenCode services;

        /// <summary>
        ///     当前彩种
        /// </summary>
        private SCCLottery currentLottery => SCCLottery.HongKongSMHLHC;

        /// <summary>
        ///     邮件接口
        /// </summary>
        private IEmail email;

        /// <summary>
        ///     是否本次运行抓取到开奖数据
        /// </summary>
        private bool isGetData;

      
    }
}