﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HtmlAgilityPack;
using Quartz;
using SCC.Common;
using SCC.Crawler.Tools;
using SCC.Interface;
using SCC.Models;

namespace SCC.Crawler.DT
{
    /// <summary>
    ///     辽宁风采35选7
    /// </summary>
    [DisallowConcurrentExecution]
    [PersistJobDataAfterExecution]
    public class LiaoNingFC35X7Job : IJob
    {
        public LiaoNingFC35X7Job()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
            email = IOC.Resolve<IEmail>();
        }

        
 
        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);


            var UrlList = GetKaiJiangUrl(Config.Number);
            foreach (var list in UrlList)
            {
                var OpenList = GetOpenListFromBackUrl(list.Url);
                foreach (var openList in OpenList)
                {
                    LatestItem = openList;
                    if (LatestItem != null && services.AddDTOpen8Code(currentLottery, LatestItem))
                    {
                        log.Info(GetType(), CommonHelper.GetJobMainLogInfo(Config, openList.Term.ToString()));
                        isGetData = true;

                    }

                }       
                context.JobDetail.JobDataMap["LatestItem"] = LatestItem;
            }
        }


        private List<OpenCode8DTModel> GetOpenListFromBackUrl(string Url)
        {
            var result = new List<OpenCode8DTModel>();
            try
            {
                var url = string.Format(Url);
                var htmlResource = NetHelper.GetUrlResponse(url, Encoding.Default);
                if (!string.IsNullOrEmpty(htmlResource))
                {
                    var doc = new HtmlDocument();
                    doc.LoadHtml(htmlResource);
                    var tables = doc.DocumentNode.SelectNodes("//table[@class]");
                    foreach (var item in tables[0].ChildNodes)
                        if (item.Name == "tr")
                            if (item.GetAttributeValue("class", "") == "")
                            {
                                var tds = item.ChildNodes.Select(w => w).ToList();
                                tds.RemoveAll(w => w.Name != "td");
                                if (tds.Count != 10) continue;
                                var qihao = tds[0].InnerText;
                                var code = tds[1].InnerText;
                                var optime = tds[9].InnerText;
                                if (code.Length > 6)
                                {
                                    var tmp = new OpenCode8DTModel
                                    {
                                        Term = long.Parse(qihao),
                                        OpenTime = DateTime.Parse(optime)
                                    };
                                    tmp.OpenCode1 = int.Parse(code.Substring(0, 2));
                                    tmp.OpenCode2 = int.Parse(code.Substring(2, 2));
                                    tmp.OpenCode3 = int.Parse(code.Substring(4, 2));
                                    tmp.OpenCode4 = int.Parse(code.Substring(6, 2));
                                    tmp.OpenCode5 = int.Parse(code.Substring(8, 2));
                                    tmp.OpenCode6 = int.Parse(code.Substring(10, 2));
                                    tmp.OpenCode7 = int.Parse(code.Substring(12, 2));
                                    tmp.OpenCode8 = int.Parse(code.Substring(14));
                                    var entity = new KaijiangDetailsEntity();
                                    entity.KaiJiangItems = new List<Kaijiangitem>();
                                    var xiaoshoue = string.IsNullOrEmpty(tds[2].InnerText.Replace(",", "").Replace("元", "").Trim()) ? "0" : tds[2].InnerText.Replace(",", "").Replace("元", "").Trim();

                                    var yidengjiang = new Kaijiangitem
                                    {
                                        Name = "一等奖",

                                        Total = string.IsNullOrEmpty(tds[3].InnerHtml.Trim()) ? "0" : tds[3].InnerHtml.Trim(),
                                        TotalMoney = string.IsNullOrEmpty(tds[4].InnerHtml.Trim()) ? "0" : tds[4].InnerHtml.Trim()

                                    };
                                    entity.KaiJiangItems.Add(yidengjiang);
                                    var erdengjiang = new Kaijiangitem
                                    {
                                        Name = "二等奖",
                                        Total = string.IsNullOrEmpty(tds[5].InnerHtml.Trim()) ? "0" : tds[5].InnerHtml.Trim(),
                                        TotalMoney = string.IsNullOrEmpty(tds[6].InnerHtml.Trim()) ? "0" : tds[6].InnerHtml.Trim()
                                    };
                                    entity.KaiJiangItems.Add(erdengjiang);
                                    var sandengjiang = new Kaijiangitem
                                    {
                                        Name = "三等奖",
                                        Total = string.IsNullOrEmpty(tds[7].InnerHtml.Trim()) ? "0" : tds[7].InnerHtml.Trim(),
                                        TotalMoney = string.IsNullOrEmpty(tds[8].InnerHtml.Trim()) ? "0" : tds[8].InnerHtml.Trim()
                                    };
                                    entity.KaiJiangItems.Add(sandengjiang);
                                    entity.Trje = xiaoshoue;
                                    entity.Gdje = "0";
                                    tmp.Spare = entity.TryToJson();
                                    result.Add(tmp);
                                }
                            }
                }
            }
            catch (Exception ex)
            {
                log.Error(GetType(),
                    string.Format("【{0}】通过备用站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + currentLottery, ex.Message));
            }

            return result;
        }

        private List<Periods> GetKaiJiangUrl(string Number)
        {
            List<Periods> list = new List<Periods>();
            string MianUrl = Config.MainUrl;
            for (int i = 1; i <= int.Parse(Number); i++)
            {
                Periods ssq = new Periods();
                if (i == 1)
                    ssq.Url = MianUrl + "?lid=10051";
                else
                    ssq.Url = MianUrl + "?lid=10051&page=" + i;
                list.Add(ssq);
            }
            return list;
        }


        #region Attribute

        /// <summary>
        ///     配置信息
        /// </summary>
        private SCCConfig Config;

        /// <summary>
        ///     当天抓取的最新一期开奖记录
        /// </summary>
        private OpenCode8DTModel LatestItem;
        

        /// <summary>
        ///     日志对象
        /// </summary>
        private readonly LogHelper log;

        /// <summary>
        ///     数据服务
        /// </summary>
        private readonly IDTOpenCode services;

        /// <summary>
        ///     当前彩种
        /// </summary>
        private SCCLottery currentLottery => SCCLottery.LiaoNingFC35X7;

        /// <summary>
        ///     邮件接口
        /// </summary>
        private IEmail email;

        /// <summary>
        ///     是否本次运行抓取到开奖数据
        /// </summary>
        private bool isGetData;

        #endregion
    }
}