﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using HtmlAgilityPack;
using Quartz;
using SCC.Common;
using SCC.Crawler.Tools;
using SCC.Interface;
using SCC.Models;

namespace SCC.Crawler.DT
{
    /// <summary>
    ///     浙江体彩6+1
    /// </summary>
    [DisallowConcurrentExecution]
    [PersistJobDataAfterExecution]
    public class ZJTC6J1Job : IJob
    {
        public ZJTC6J1Job()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
            email = IOC.Resolve<IEmail>();
        }

        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);


            var UrlList = GetKaiJiangUrl(Config.Number);
            foreach (var list in UrlList)
            {
                var OpenList = GetOpenListFromMainUrl(list.Url);
                foreach (var openList in OpenList)
                {
                    LatestItem = openList;
                    if (LatestItem != null && services.AddDTOpen7Code(currentLottery, LatestItem))
                    {
                        log.Info(GetType(), CommonHelper.GetJobMainLogInfo(Config, openList.Term.ToString()));
                        isGetData = true;

                    }

                }
                //}


                ////保存最新期号
                context.JobDetail.JobDataMap["LatestItem"] = LatestItem;





            }
        }
        private List<Periods> GetKaiJiangUrl(string Number)
        {
            List<Periods> list = new List<Periods>();
            string MianUrl = Config.MainUrl;
            for (int i = 1; i <= int.Parse(Number); i++)
            {
                Periods ssq = new Periods();
                if (i == 1)
                    ssq.Url = MianUrl + "historykj/history.jspx?_ltype=zj61";
                else
                    ssq.Url = MianUrl + "historykj/history_" + i+ ".jspx?_ltype=zj61";
                list.Add(ssq);
            }
            return list;
        }
        private List<OpenCode7DTModel> GetOpenListFromMainUrl(string MainUrl)
        {
            var result = new List<OpenCode7DTModel>();
            try
            {
                var htmlResource = NetHelper.GetUrlResponse(MainUrl, Encoding.UTF8);
                if (htmlResource == null) return result;

                var doc = new HtmlDocument();
                doc.LoadHtml(htmlResource);
                var tbody = doc.DocumentNode.SelectSingleNode("//tbody");
                if (tbody == null) return result;
                var trs = tbody.ChildNodes.Where(N => N.Name.ToLower() == "tr").ToList();
                foreach (var item in trs)
                {
                    var tds = item.ChildNodes.Where(w => w.Name == "td").ToList();
                    var qihao = tds[0].InnerText.Trim();
                    var kaijianghao = tds[1].InnerText.Trim().Replace(" ", "");
                    var opentime = tds[10].InnerText.Trim();
                    var tmp = new OpenCode7DTModel
                    {
                        Term = long.Parse(qihao),
                        OpenTime = DateTime.Parse(opentime)
                    };
                    tmp.OpenCode1 = int.Parse(kaijianghao.Substring(0, 1));
                    tmp.OpenCode2 = int.Parse(kaijianghao.Substring(1, 1));
                    tmp.OpenCode3 = int.Parse(kaijianghao.Substring(2, 1));
                    tmp.OpenCode4 = int.Parse(kaijianghao.Substring(3, 1));
                    tmp.OpenCode5 = int.Parse(kaijianghao.Substring(4, 1));
                    tmp.OpenCode6 = int.Parse(kaijianghao.Substring(5, 1));
                    tmp.OpenCode7 = int.Parse(kaijianghao.Substring(6, 1));
                    OptimizeMainModel(ref tmp, item);
                    result.Add(tmp);
                }

            }
            catch (Exception ex)
            {
                log.Error(GetType(),
                    string.Format("【{0}】通过主站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + currentLottery, ex.Message));
            }

            return result;
        }

        /// <summary>
        ///     完善主站浙江体彩6+1开奖详情信息
        /// </summary>
        /// <param name="model"></param>
        private bool OptimizeMainModel(ref OpenCode7DTModel model, HtmlNode tr)
        {
            try
            {
                var tds = tr.ChildNodes.Where(w => w.Name == "td").ToList();
                var entity = new KaijiangDetailsEntity();
                entity.KaiJiangItems = new List<Kaijiangitem>();
                var xiaoshoue = tds[8].InnerText.Replace("- -","0").Trim();
                var jiangchi = (double.Parse(tds[9].InnerText.Replace("- -", "0").Trim()) * 10000).ToString();
                var tedengjiangary = tds[2].InnerText.Trim().Split('注', '元');

                var tedengjiang = new Kaijiangitem
                {
                    Name = "特等奖",
                    Total = tedengjiangary[0].Replace(",", string.Empty),
                    TotalMoney = tedengjiangary[1].Replace(",", string.Empty).Replace("\t", "")
                };
                entity.KaiJiangItems.Add(tedengjiang);
                var yidengjiangary = tds[3].InnerText.Trim().Split('注', '元');
                var yidengjiang = new Kaijiangitem
                {
                    Name = "一等奖",
                    Total = yidengjiangary[0].Replace(",", string.Empty),
                    TotalMoney = yidengjiangary[1].Replace(",", string.Empty).Replace("\t", "")
                };
                entity.KaiJiangItems.Add(yidengjiang);
                var erdengjiangary = tds[4].InnerText.Trim().Split('注', '元');
                var erdengjiang = new Kaijiangitem
                {
                    Name = "二等奖",
                    Total = erdengjiangary[0].Replace(",", string.Empty),
                    TotalMoney = erdengjiangary[1].Replace(",", string.Empty).Replace("t\\", "")
                };
                entity.KaiJiangItems.Add(erdengjiang);
                var sandengjiangary = tds[5].InnerText.Trim().Split('注', '元');
                var sandengjiang = new Kaijiangitem
                {
                    Name = "三等奖",
                    Total = sandengjiangary[0].Replace(",", string.Empty),
                    TotalMoney = sandengjiangary[1].Replace(",", string.Empty).Replace("\t", "")
                };
                entity.KaiJiangItems.Add(sandengjiang);
                var sidengjiangary = tds[6].InnerText.Trim().Split('注', '元');
                var sidengjiang = new Kaijiangitem
                {
                    Name = "四等奖",
                    Total = sidengjiangary[0].Replace(",", string.Empty),
                    TotalMoney = sidengjiangary[1].Replace(",", string.Empty).Replace("\t", "")
                };
                entity.KaiJiangItems.Add(sidengjiang);
                var wudengjiangary = tds[7].InnerText.Trim().Split('注', '元');
                var wudengjiang = new Kaijiangitem
                {
                    Name = "五等奖",
                    Total = wudengjiangary[0].Replace(",", string.Empty),
                    TotalMoney = wudengjiangary[1].Replace(",", string.Empty).Replace("\t", "")
                };
                entity.KaiJiangItems.Add(wudengjiang);
                model.Spare = entity.TryToJson();
            }
            catch (Exception ex)
            {
                log.Error(GetType(),
                    string.Format("【{0}】通过主站点优化开奖列表时发生错误，错误信息【{1}】", Config.Area + currentLottery, ex.Message));
            }

            return false;
        }

  

        #region Attribute

        /// <summary>
        ///     配置信息
        /// </summary>
        private SCCConfig Config;

        /// <summary>
        ///     当天抓取的最新一期开奖记录
        /// </summary>
        private OpenCode7DTModel LatestItem;

        /// <summary>
        ///     当天抓取失败列表
        /// </summary>
        private List<string> FailedQiHaoList;

        /// <summary>
        ///     日志对象
        /// </summary>
        private readonly LogHelper log;

        /// <summary>
        ///     数据服务
        /// </summary>
        private readonly IDTOpenCode services;

        /// <summary>
        ///     当前彩种
        /// </summary>
        private SCCLottery currentLottery => SCCLottery.ZheJiang6J1;

        /// <summary>
        ///     邮件接口
        /// </summary>
        private readonly IEmail email;

        /// <summary>
        ///     是否本次运行抓取到开奖数据
        /// </summary>
        private bool isGetData;

        #endregion
    }
}