﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HtmlAgilityPack;
using Quartz;
using SCC.Common;
using SCC.Crawler.Tools;
using SCC.Interface;
using SCC.Models;

namespace SCC.Crawler.QG
{
    /// <summary>
    ///     福彩3D
    /// </summary>
    public class FC3DJob : IJob
    {
        public FC3DJob()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
            email = IOC.Resolve<IEmail>();
        }

        /// <summary>
        ///     作业执行入口
        /// </summary>
        /// <param name="context">作业执行上下文</param>
        public void Execute(IJobExecutionContext context)
        {
           
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
            string years = Config.Year;
            string[] yearArr = years.Split(",".ToCharArray());
            if (yearArr.Length > 0)
            {
                foreach (string y in yearArr)
                {
                    var qihao = GetQiHaoList(y, Config.Number);
                    foreach (var list in qihao)
                    {
                        LatestItem = GetOpenListFromMainUrl(list.QiHao.ToString());
                        if (LatestItem != null && services.AddDTOpenFC3DCode(currentLottery, LatestItem))
                        {
                            log.Info(GetType(), CommonHelper.GetJobMainLogInfo(Config, list.QiHao.ToString()));
                            isGetData = true;
                        }
                    }
                }
            }

        
            //保存最新期号
            context.JobDetail.JobDataMap["LatestItem"] = LatestItem;
        }
        /// <summary>
        ///     通过主站点爬取开奖数据
        ///     (福建体彩网)
        /// </summary>
        private OpenCodeFC3DTModel GetOpenListFromMainUrl(string qishu)
        {
            var result = new OpenCodeFC3DTModel();
            try
            {
                var URL = "https://www.8200.cn/kjh/3d/" + qishu + ".htm";
                HtmlNode node = null;
                var htmlResource = NetHelper.GetUrlResponse(URL, Encoding.GetEncoding("utf-8"));
                if (htmlResource == null) return result;

                var doc = new HtmlDocument();
                node = doc.DocumentNode;
                doc.LoadHtml(htmlResource);
                var div = doc.DocumentNode.SelectSingleNode("//div[@class='ballBox ball40 mb-20 clearfix']");
                if (div == null) return null;
                var kaijianghao = div.InnerText.Replace(" ", "").Trim();
                result.OpenCode1 = int.Parse(kaijianghao.Substring(0, 1));
                result.OpenCode2 = int.Parse(kaijianghao.Substring(1, 1));
                result.OpenCode3 = int.Parse(kaijianghao.Substring(2, 1));
                var Time = node.SelectSingleNode("//div[@class='text-16']");
                if (div == null) return null;

                //爬去時間
                var p = Time.ChildNodes.Where(nodes => nodes.Name == "p").ToList();
                result.Term = long.Parse(qishu);
                result.KaiJiHao = p[0].InnerText.Substring(14,3).Insert(1, ",").Insert(3, ",");
                result.ShiJiHao = p[0].InnerText.Substring(23,3).Insert(1, ",").Insert(3, ",");
                var kaijiangTime = p[2].InnerText.Substring(5, 10);
                result.OpenTime = Convert.ToDateTime(kaijiangTime);


                var details = GetKaijiangDetails(node);
                result.Spare = details;





            }
            catch (Exception ex)
            {
                log.Error(GetType(),
                    string.Format("【{0}】通过主站抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + currentLottery, ex.Message));
            }

            return result;
        }

        private string GetKaijiangDetails(HtmlNode nodes)
        {


            var div = nodes.SelectSingleNode("//div[@class='text-16']");
            if (div == null) return null;

            //爬去奖金
            var jiangjin = div.ChildNodes.Where(node => node.Name == "p").ToList();


            //爬去奖项
            //var tbody = div.ChildNodes.Where(node => node.Name == "tbody").ToList();
            var table = nodes.SelectSingleNode("//table");
            var trs = table.ChildNodes.Where(node => node.Name == "tr").ToList();

            var trje = jiangjin[3].InnerText.Replace(" 万元", "").Replace(" 亿元", "").Replace("本期销量：", "").Replace("--", "0").Replace(",", "")
                .Trim();


            var entity = new KaijiangDetailsEntity
            {
                Gdje = null,
                Trje = trje == "0" ? "0" : (double.Parse(trje) * 10000).ToString()
            };
            //TODO 

            //组装详情  
            var list = new List<Kaijiangitem>();
            for (var i = 0; i < trs.Count; i++)
            {
                var tds = trs[i].ChildNodes.Where(node => node.Name == "td").ToList();


                var kaijiangitem = new Kaijiangitem();

                var TotalMoney = tds[1].InnerText.Replace("元", "").Replace("--", "0").Replace(",", "").Trim();

                kaijiangitem.Name = tds[0].InnerText.Trim();
                kaijiangitem.TotalMoney = TotalMoney == "0" ? "0" : double.Parse(TotalMoney).ToString();
                kaijiangitem.Total = tds[2].InnerText.Trim().Replace(" 注", "").Replace("--", "0").Trim();
                list.Add(kaijiangitem);
            }

            entity.KaiJiangItems = list;

            return entity.TryToJson();
        }

        public List<Periods> GetQiHaoList(string QiHao, string sum)
        {
            List<Periods> list = new List<Periods>();
            int StartQIhao = int.Parse(QiHao + "001");
            int AtLast = int.Parse(QiHao + sum);
            for (int i = StartQIhao; i <= AtLast; i++)
            {
                Periods ssq = new Periods();
                ssq.QiHao = i;
                list.Add(ssq);
            }

            return list;

        }

        #region Attribute

        /// <summary>
        ///     配置信息
        /// </summary>
        private SCCConfig Config;

        /// <summary>
        ///     当天抓取的最新一期开奖记录
        /// </summary>
        private OpenCodeFC3DTModel LatestItem;

        /// <summary>
        ///     当天抓取失败列表
        /// </summary>
        private List<string> FailedQiHaoList = null;

        /// <summary>
        ///     日志对象
        /// </summary>
        private readonly LogHelper log;

        /// <summary>
        ///     数据服务
        /// </summary>
        private readonly IDTOpenCode services;

        /// <summary>
        ///     当前彩种
        /// </summary>
        private SCCLottery currentLottery => SCCLottery.FC3D;

        /// <summary>
        ///     邮件接口
        /// </summary>
        private IEmail email;

        /// <summary>
        ///     是否本次运行抓取到开奖数据
        /// </summary>
        private bool isGetData;

        #endregion
    }
}