﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HtmlAgilityPack;
using Quartz;
using SCC.Common;
using SCC.Crawler.Tools;
using SCC.Interface;
using SCC.Models;

namespace SCC.Crawler.QG
{
    /// <summary>
    /// 七星彩
    /// </summary>
    public class QXCJob : IJob
    {
        /// <summary>
        /// 构造函数
        /// </summary>
        public QXCJob()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
            email = IOC.Resolve<IEmail>();
        }
        /// <summary>
        ///  七星彩作业执行入口
        /// </summary>
        /// <param name="context">作业执行上下文</param>
        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
            string years = Config.Year;
            string[] yearArr = years.Split(",".ToCharArray());
            if (yearArr.Length > 0)
            {
                foreach (string y in yearArr)
                {
                    var qihao = GetQiHaoList(y, Config.Number);
                    foreach (var list in qihao)
                    {
                        LatestItem = GetOpenListFromMainUrl(list.QiHao.ToString());
                     
                        if (LatestItem != null && services.AddDTOpen7Code(currentLottery, LatestItem))
                        {
                            log.Info(GetType(), CommonHelper.GetJobMainLogInfo(Config, list.QiHao.ToString()));
                            isGetData = true;
                        }
                    }
                }
            }
            //保存最新期号
            context.JobDetail.JobDataMap["LatestItem"] = LatestItem;
        }
        /// <summary>
        /// 根据期号通过站点爬取开奖数据
        /// </summary>
        private OpenCode7DTModel GetOpenListFromMainUrl(string qishu)
        {
            var result = new OpenCode7DTModel();
            try
            {
                var URL = "https://www.8200.cn/kjh/qxc/" + qishu + ".htm";
                HtmlNode node = null;
                var htmlResource = NetHelper.GetUrlResponse(URL, Encoding.GetEncoding("utf-8"));
                if (htmlResource == null) return result;
                var doc = new HtmlDocument();
                node = doc.DocumentNode;
                doc.LoadHtml(htmlResource);
                var div = doc.DocumentNode.SelectSingleNode("//div[@class='ballBox ball40 mb-20 clearfix']");
                if (div == null) return null;
                var kaijianghao = div.InnerText.Replace(" ", "").Trim();
                result.OpenCode1 = int.Parse(kaijianghao.Substring(0, 1));
                result.OpenCode2 = int.Parse(kaijianghao.Substring(1, 1));
                result.OpenCode3 = int.Parse(kaijianghao.Substring(2, 1));
                result.OpenCode4 = int.Parse(kaijianghao.Substring(3, 1));
                result.OpenCode5 = int.Parse(kaijianghao.Substring(4, 1));
                result.OpenCode6 = int.Parse(kaijianghao.Substring(5, 1));
                result.OpenCode7 = int.Parse(kaijianghao.Substring(6, 1));
                var Time = node.SelectSingleNode("//div[@class='text-16']");
        
                //爬去時間
                var p = Time.ChildNodes.Where(nodes => nodes.Name == "p").ToList();
                result.Term = long.Parse(qishu.Substring(2));
                var kaijiangTime = p[0].InnerText.Substring(5, 10);
                result.OpenTime = Convert.ToDateTime(kaijiangTime);
                var details = GetKaijiangDetails(node);
                result.Spare = details;
            }
            catch (Exception ex)
            {
                log.Error(GetType(),
                    string.Format("【{0}】通过网站抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + currentLottery, ex.Message));
            }
            return result;
        }
        /// <summary>
        /// 获得该期的详情
        /// </summary>
        /// <param name="nodes">html内容</param>
        /// <returns></returns>
        private string GetKaijiangDetails(HtmlNode nodes)
        {
            var div = nodes.SelectSingleNode("//div[@class='text-16']");
            if (div == null) return null;
            //爬去奖金
            var jiangjin = div.ChildNodes.Where(node => node.Name == "p").ToList();
            //爬去奖项
            //var tbody = div.ChildNodes.Where(node => node.Name == "tbody").ToList();
            var table = nodes.SelectSingleNode("//table");
            var trs = table.ChildNodes.Where(node => node.Name == "tr").ToList();
            var gdje = jiangjin[2].InnerText.Replace(" 万元", "").Replace(" 亿元", "").Replace("奖池滚存：", "").Replace("--", "0").Replace(",", "")
                .Trim();
            var trje = jiangjin[1].InnerText.Replace(" 万元", "").Replace(" 亿元", "").Replace("本期销量：", "").Replace("--", "0").Replace(",", "")
                .Trim();
            var entity = new KaijiangDetailsEntity
            {
                Gdje = gdje == "0" ? "0" : (double.Parse(gdje) * 10000).ToString(),
                Trje = trje == "0" ? "0" : (double.Parse(trje) * 10000).ToString()
            };
            //组装详情  
            var list = new List<Kaijiangitem>();
            for (var i = 0; i < trs.Count; i++)
            {
                var tds = trs[i].ChildNodes.Where(node => node.Name == "td").ToList();
                var kaijiangitem = new Kaijiangitem();
                var TotalMoney = tds[1].InnerText.Replace("元", "").Replace("--", "0").Replace(",", "").Trim();
                kaijiangitem.Name = tds[0].InnerText.Trim();
                kaijiangitem.TotalMoney = TotalMoney == "0" ? "0" : double.Parse(TotalMoney).ToString();
                kaijiangitem.Total = tds[2].InnerText.Trim().Replace(" 注", "").Replace("--", "0").Trim();
                list.Add(kaijiangitem);
            }
            entity.KaiJiangItems = list;
            return entity.TryToJson();
        }
        /// <summary>
        /// 获取开始和结束的期号列表
        /// </summary>
        /// <param name="Year">年</param>
        /// <param name="sum">每年多少期</param>
        /// <returns></returns>
        public List<Periods> GetQiHaoList(string Year, string Number)
        {
            List<Periods> list = new List<Periods>();
            int StartQIhao = int.Parse(Year + "001");
            int AtLast = int.Parse(Year + Number);
            for (int i = StartQIhao; i <= AtLast; i++)
            {
                Periods ssq = new Periods();
                ssq.QiHao = i;
                list.Add(ssq);
            }
            return list;
        }
        #region Attribute

        /// <summary>
        ///     配置信息
        /// </summary>
        private SCCConfig Config;

        /// <summary>
        ///     当天抓取的最新一期开奖记录
        /// </summary>
        private OpenCode7DTModel LatestItem;

        /// <summary>
        ///     当天抓取失败列表
        /// </summary>
        private List<string> FailedQiHaoList = null;

        /// <summary>
        ///     日志对象
        /// </summary>
        private readonly LogHelper log;

        /// <summary>
        ///     数据服务
        /// </summary>
        private readonly IDTOpenCode services;

        /// <summary>
        ///     当前彩种
        /// </summary>
        private SCCLottery currentLottery => SCCLottery.QXC;

        /// <summary>
        ///     邮件接口
        /// </summary>
        private IEmail email;

        /// <summary>
        ///     是否本次运行抓取到开奖数据
        /// </summary>
        private bool isGetData;

        #endregion
    }
}